@@ -1,5 +1,5 @@
 /*****************************************************************************
- * 
+ *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
  * Copyright (C) 2006 Artem Pavlenko, Jean-Francois Doyon
@@ -19,13 +19,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *****************************************************************************/
-//$Id: mapnik_python.cc 27 2005-03-30 21:45:40Z pavlenko $
 
 #include <boost/python.hpp>
 #include <boost/get_pointer.hpp>
 #include <boost/python/detail/api_placeholder.hpp>
 #include <boost/python/exception_translator.hpp>
 
+// stl
+#include <stdexcept>
+
 void register_cairo();
 void export_color();
 void export_coord();
@@ -37,6 +39,8 @@ void export_geometry();
 void export_palette();
 void export_image();
 void export_image_view();
+void export_gamma_method();
+void export_scaling_method();
 void export_grid();
 void export_grid_view();
 void export_map();
@@ -47,6 +51,7 @@ void export_style();
 void export_stroke();
 void export_feature();
 void export_featureset();
+void export_fontset();
 void export_datasource();
 void export_datasource_cache();
 void export_symbolizer();
@@ -55,74 +60,92 @@ void export_point_symbolizer();
 void export_line_symbolizer();
 void export_line_pattern_symbolizer();
 void export_polygon_symbolizer();
+void export_building_symbolizer();
 void export_polygon_pattern_symbolizer();
 void export_raster_symbolizer();
-void export_text_symbolizer();
+void export_text_placement();
 void export_shield_symbolizer();
+void export_debug_symbolizer();
 void export_font_engine();
 void export_projection();
 void export_proj_transform();
 void export_view_transform();
 void export_raster_colorizer();
-void export_glyph_symbolizer();
-void export_inmem_metawriter();
+void export_label_collision_detector();
+void export_logger();
 
 #include <mapnik/version.hpp>
 #include <mapnik/value_error.hpp>
+#include <mapnik/layer.hpp>
 #include <mapnik/map.hpp>
 #include <mapnik/agg_renderer.hpp>
 #ifdef HAVE_CAIRO
 #include <mapnik/cairo_renderer.hpp>
 #endif
 #include <mapnik/graphics.hpp>
+#include <mapnik/stroke.hpp>
+#include <mapnik/font_set.hpp>
+#include <mapnik/rule.hpp>
 #include <mapnik/image_util.hpp>
 #include <mapnik/load_map.hpp>
-#include <mapnik/config_error.hpp>
 #include <mapnik/scale_denominator.hpp>
 #include <mapnik/value_error.hpp>
 #include <mapnik/save_map.hpp>
+#include <mapnik/scale_denominator.hpp>
 #include "python_grid_utils.hpp"
+#include "mapnik_value_converter.hpp"
+#include "mapnik_threads.hpp"
+#include "python_optional.hpp"
+#include <mapnik/marker_cache.hpp>
+#include <mapnik/mapped_memory_cache.hpp>
+
+
+void clear_cache()
+{
+    mapnik::marker_cache::instance().clear();
+    mapnik::mapped_memory_cache::instance().clear();
+}
 
 #if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
 #include <pycairo.h>
 static Pycairo_CAPI_t *Pycairo_CAPI;
 #endif
 
-
-namespace boost { namespace python {
-
-    struct mapnik_value_to_python
-    {
-        static PyObject* convert(mapnik::value const& v)
-        {
-            return boost::apply_visitor(value_converter(),v.base());
-        }
-    };
-}}
+using mapnik::python_thread;
+using mapnik::python_unblock_auto_block;
+#ifdef MAPNIK_DEBUG
+bool python_thread::thread_support = true;
+#endif
+boost::thread_specific_ptr<PyThreadState> python_thread::state;
 
 void render(const mapnik::Map& map,
-    mapnik::image_32& image,
+            mapnik::image_32& image,
+            double scale_factor = 1.0,
+            unsigned offset_x = 0u,
+            unsigned offset_y = 0u)
+{
+    python_unblock_auto_block b;
+    mapnik::agg_renderer<mapnik::image_32> ren(map,image,scale_factor,offset_x, offset_y);
+    ren.apply();
+
+}
+
+void render_with_detector(
+    const mapnik::Map &map,
+    mapnik::image_32 &image,
+    boost::shared_ptr<mapnik::label_collision_detector4> detector,
     double scale_factor = 1.0,
     unsigned offset_x = 0u,
     unsigned offset_y = 0u)
 {
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        mapnik::agg_renderer<mapnik::image_32> ren(map,image,scale_factor,offset_x, offset_y);
-        ren.apply();
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::agg_renderer<mapnik::image_32> ren(map,image,detector);
+    ren.apply();
 }
 
 void render_layer2(const mapnik::Map& map,
-    mapnik::image_32& image,
-    unsigned layer_idx)
+                   mapnik::image_32& image,
+                   unsigned layer_idx)
 {
     std::vector<mapnik::layer> const& layers = map.layers();
     std::size_t layer_num = layers.size();
@@ -133,106 +156,113 @@ void render_layer2(const mapnik::Map& map,
         throw std::runtime_error(s.str());
     }
 
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        mapnik::layer const& layer = layers[layer_idx];
-        mapnik::agg_renderer<mapnik::image_32> ren(map,image,1.0,0,0);
-        std::set<std::string> names;
-        ren.apply(layer,names);
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::layer const& layer = layers[layer_idx];
+    mapnik::agg_renderer<mapnik::image_32> ren(map,image,1.0,0,0);
+    std::set<std::string> names;
+    ren.apply(layer,names);
 }
 
 #if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
 
 void render3(const mapnik::Map& map,
-    PycairoSurface* surface,
-    unsigned offset_x = 0,
-    unsigned offset_y = 0)
+             PycairoSurface* py_surface,
+             double scale_factor = 1.0,
+             unsigned offset_x = 0,
+             unsigned offset_y = 0)
 {
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        Cairo::RefPtr<Cairo::Surface> s(new Cairo::Surface(surface->surface));
-        mapnik::cairo_renderer<Cairo::Surface> ren(map,s,offset_x, offset_y);
-        ren.apply();
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::cairo_surface_ptr surface(cairo_surface_reference(py_surface->surface), mapnik::cairo_surface_closer());
+    mapnik::cairo_renderer<mapnik::cairo_surface_ptr> ren(map,surface,scale_factor,offset_x,offset_y);
+    ren.apply();
 }
 
-void render4(const mapnik::Map& map, PycairoSurface* surface)
+void render4(const mapnik::Map& map, PycairoSurface* py_surface)
 {
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        Cairo::RefPtr<Cairo::Surface> s(new Cairo::Surface(surface->surface));
-        mapnik::cairo_renderer<Cairo::Surface> ren(map,s);
-        ren.apply();
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::cairo_surface_ptr surface(cairo_surface_reference(py_surface->surface), mapnik::cairo_surface_closer());
+    mapnik::cairo_renderer<mapnik::cairo_surface_ptr> ren(map,surface);
+    ren.apply();
 }
 
 void render5(const mapnik::Map& map,
-    PycairoContext* context,
-    unsigned offset_x = 0,
-    unsigned offset_y = 0)
+             PycairoContext* py_context,
+             double scale_factor = 1.0,
+             unsigned offset_x = 0,
+             unsigned offset_y = 0)
 {
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        Cairo::RefPtr<Cairo::Context> c(new Cairo::Context(context->ctx));
-        mapnik::cairo_renderer<Cairo::Context> ren(map,c,offset_x, offset_y);
-        ren.apply();
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::cairo_ptr context(py_context->ctx, mapnik::cairo_closer());
+    mapnik::cairo_renderer<mapnik::cairo_ptr> ren(map,context,scale_factor,offset_x, offset_y);
+    ren.apply();
 }
 
-void render6(const mapnik::Map& map, PycairoContext* context)
+void render6(const mapnik::Map& map, PycairoContext* py_context)
 {
-    Py_BEGIN_ALLOW_THREADS
-    try
-    {
-        Cairo::RefPtr<Cairo::Context> c(new Cairo::Context(context->ctx));
-        mapnik::cairo_renderer<Cairo::Context> ren(map,c);
-        ren.apply();
-    }
-    catch (...)
-    {
-        Py_BLOCK_THREADS
-        throw;
-    }
-    Py_END_ALLOW_THREADS
+    python_unblock_auto_block b;
+    mapnik::cairo_ptr context(py_context->ctx, mapnik::cairo_closer());
+    mapnik::cairo_renderer<mapnik::cairo_ptr> ren(map,context);
+    ren.apply();
+}
+
+void render_with_detector2(
+    const mapnik::Map& map,
+    PycairoContext* py_context,
+    boost::shared_ptr<mapnik::label_collision_detector4> detector)
+{
+    python_unblock_auto_block b;
+    mapnik::cairo_ptr context(py_context->ctx, mapnik::cairo_closer());
+    mapnik::cairo_renderer<mapnik::cairo_ptr> ren(map,context,detector);
+    ren.apply();
+}
+
+void render_with_detector3(
+    const mapnik::Map& map,
+    PycairoContext* py_context,
+    boost::shared_ptr<mapnik::label_collision_detector4> detector,
+    double scale_factor = 1.0,
+    unsigned offset_x = 0u,
+    unsigned offset_y = 0u)
+{
+    python_unblock_auto_block b;
+    mapnik::cairo_ptr context(py_context->ctx, mapnik::cairo_closer());
+    mapnik::cairo_renderer<mapnik::cairo_ptr> ren(map,context,detector,scale_factor,offset_x,offset_y);
+    ren.apply();
+}
+
+void render_with_detector4(
+    const mapnik::Map& map,
+    PycairoSurface* py_surface,
+    boost::shared_ptr<mapnik::label_collision_detector4> detector)
+{
+    python_unblock_auto_block b;
+    mapnik::cairo_surface_ptr surface(cairo_surface_reference(py_surface->surface), mapnik::cairo_surface_closer());
+    mapnik::cairo_renderer<mapnik::cairo_surface_ptr> ren(map, surface, detector);
+    ren.apply();
+}
+
+void render_with_detector5(
+    const mapnik::Map& map,
+    PycairoSurface* py_surface,
+    boost::shared_ptr<mapnik::label_collision_detector4> detector,
+    double scale_factor = 1.0,
+    unsigned offset_x = 0u,
+    unsigned offset_y = 0u)
+{
+    python_unblock_auto_block b;
+    mapnik::cairo_surface_ptr surface(cairo_surface_reference(py_surface->surface), mapnik::cairo_surface_closer());
+    mapnik::cairo_renderer<mapnik::cairo_surface_ptr> ren(map, surface, detector, scale_factor, offset_x, offset_y);
+    ren.apply();
 }
 
 #endif
 
 
-void render_tile_to_file(const mapnik::Map& map, 
+void render_tile_to_file(const mapnik::Map& map,
                          unsigned offset_x, unsigned offset_y,
                          unsigned width, unsigned height,
-                         const std::string& file,
-                         const std::string& format)
+                         std::string const& file,
+                         std::string const& format)
 {
     mapnik::image_32 image(width,height);
     render(map,image,1.0,offset_x, offset_y);
@@ -240,78 +270,90 @@ void render_tile_to_file(const mapnik::Map& map,
 }
 
 void render_to_file1(const mapnik::Map& map,
-                     const std::string& filename,
-                     const std::string& format)
+                     std::string const& filename,
+                     std::string const& format)
 {
     if (format == "pdf" || format == "svg" || format =="ps" || format == "ARGB32" || format == "RGB24")
     {
 #if defined(HAVE_CAIRO)
-        mapnik::save_to_cairo_file(map,filename,format);
+        mapnik::save_to_cairo_file(map,filename,format,1.0);
 #else
         throw mapnik::ImageWriterException("Cairo backend not available, cannot write to format: " + format);
 #endif
     }
-    else 
+    else
     {
         mapnik::image_32 image(map.width(),map.height());
         render(map,image,1.0,0,0);
-        mapnik::save_to_file(image,filename,format); 
+        mapnik::save_to_file(image,filename,format);
     }
 }
 
-void render_to_file2(const mapnik::Map& map,const std::string& filename)
+void render_to_file2(const mapnik::Map& map,std::string const& filename)
 {
     std::string format = mapnik::guess_type(filename);
     if (format == "pdf" || format == "svg" || format =="ps")
     {
 #if defined(HAVE_CAIRO)
-        mapnik::save_to_cairo_file(map,filename,format);
+        mapnik::save_to_cairo_file(map,filename,format,1.0);
 #else
         throw mapnik::ImageWriterException("Cairo backend not available, cannot write to format: " + format);
 #endif
     }
-    else 
+    else
     {
         mapnik::image_32 image(map.width(),map.height());
         render(map,image,1.0,0,0);
-        mapnik::save_to_file(image,filename); 
+        mapnik::save_to_file(image,filename);
     }
 }
 
 void render_to_file3(const mapnik::Map& map,
-                     const std::string& filename,
-                     const std::string& format,
+                     std::string const& filename,
+                     std::string const& format,
                      double scale_factor = 1.0
-                     )
+    )
 {
     if (format == "pdf" || format == "svg" || format =="ps" || format == "ARGB32" || format == "RGB24")
     {
 #if defined(HAVE_CAIRO)
-        mapnik::save_to_cairo_file(map,filename,format);
+        mapnik::save_to_cairo_file(map,filename,format,scale_factor);
 #else
         throw mapnik::ImageWriterException("Cairo backend not available, cannot write to format: " + format);
 #endif
     }
-    else 
+    else
     {
         mapnik::image_32 image(map.width(),map.height());
         render(map,image,scale_factor,0,0);
-        mapnik::save_to_file(image,filename,format); 
+        mapnik::save_to_file(image,filename,format);
     }
 }
 
 double scale_denominator(mapnik::Map const &map, bool geographic)
 {
-    return mapnik::scale_denominator(map, geographic);
+    return mapnik::scale_denominator(map.scale(), geographic);
 }
 
 // http://docs.python.org/c-api/exceptions.html#standard-exceptions
-void config_error_translator(mapnik::config_error const & ex) {
+void value_error_translator(mapnik::value_error const & ex)
+{
+    PyErr_SetString(PyExc_ValueError, ex.what());
+}
+
+void runtime_error_translator(std::runtime_error const & ex)
+{
     PyErr_SetString(PyExc_RuntimeError, ex.what());
 }
 
-void value_error_translator(mapnik::value_error const & ex) {
-    PyErr_SetString(PyExc_ValueError, ex.what());
+void out_of_range_error_translator(std::out_of_range const & ex)
+{
+    PyErr_SetString(PyExc_IndexError, ex.what());
+}
+
+void standard_error_translator(std::exception const & ex)
+{
+    PyErr_SetString(PyExc_RuntimeError, ex.what());
 }
 
 unsigned mapnik_version()
@@ -319,22 +361,18 @@ unsigned mapnik_version()
     return MAPNIK_VERSION;
 }
 
-unsigned mapnik_svn_revision()
+std::string mapnik_version_string()
 {
-#if defined(SVN_REVISION)
-    return SVN_REVISION;
-#else
-    return 0;
-#endif
+    return MAPNIK_VERSION_STRING;
 }
 
 // indicator for jpeg read/write support within libmapnik
 bool has_jpeg()
 {
 #if defined(HAVE_JPEG)
-  return true;
+    return true;
 #else
-  return false;
+    return false;
 #endif
 }
 
@@ -352,14 +390,18 @@ bool has_cairo()
 bool has_pycairo()
 {
 #if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
+#if PY_MAJOR_VERSION >= 3
+    Pycairo_CAPI = (Pycairo_CAPI_t*) PyCapsule_Import(const_cast<char *>("cairo.CAPI"), 0);
+#else
     Pycairo_CAPI = (Pycairo_CAPI_t*) PyCObject_Import(const_cast<char *>("cairo"), const_cast<char *>("CAPI"));
+#endif
     if (Pycairo_CAPI == NULL){
         /*
           Case where pycairo support has been compiled into
-          mapnik but at runtime the cairo python module 
-          is unable to be imported and therefore Pycairo surfaces 
-          and contexts cannot be passed to mapnik.render() 
-        */ 
+          mapnik but at runtime the cairo python module
+          is unable to be imported and therefore Pycairo surfaces
+          and contexts cannot be passed to mapnik.render()
+        */
         return false;
     }
     return true;
@@ -369,15 +411,16 @@ bool has_pycairo()
 }
 
 
-BOOST_PYTHON_FUNCTION_OVERLOADS(load_map_overloads, load_map, 2, 3)
+BOOST_PYTHON_FUNCTION_OVERLOADS(load_map_overloads, load_map, 2, 4)
 BOOST_PYTHON_FUNCTION_OVERLOADS(load_map_string_overloads, load_map_string, 2, 4)
 BOOST_PYTHON_FUNCTION_OVERLOADS(save_map_overloads, save_map, 2, 3)
 BOOST_PYTHON_FUNCTION_OVERLOADS(save_map_to_string_overloads, save_map_to_string, 1, 2)
 BOOST_PYTHON_FUNCTION_OVERLOADS(render_overloads, render, 2, 5)
+BOOST_PYTHON_FUNCTION_OVERLOADS(render_with_detector_overloads, render_with_detector, 3, 6)
 
-BOOST_PYTHON_MODULE(_mapnik2)
+BOOST_PYTHON_MODULE(_mapnik)
 {
- 
+
     using namespace boost::python;
 
     using mapnik::load_map;
@@ -385,26 +428,31 @@ BOOST_PYTHON_MODULE(_mapnik2)
     using mapnik::save_map;
     using mapnik::save_map_to_string;
     using mapnik::render_grid;
-    
-    register_exception_translator<mapnik::config_error>(&config_error_translator);
+
+    register_exception_translator<std::exception>(&standard_error_translator);
+    register_exception_translator<std::out_of_range>(&out_of_range_error_translator);
     register_exception_translator<mapnik::value_error>(&value_error_translator);
+    register_exception_translator<std::runtime_error>(&runtime_error_translator);
     register_cairo();
     export_query();
     export_geometry();
     export_feature();
     export_featureset();
+    export_fontset();
     export_datasource();
     export_parameters();
-    export_color(); 
-    export_envelope();   
+    export_color();
+    export_envelope();
     export_palette();
     export_image();
     export_image_view();
+    export_gamma_method();
+    export_scaling_method();
     export_grid();
     export_grid_view();
     export_expression();
     export_rule();
-    export_style();    
+    export_style();
     export_layer();
     export_stroke();
     export_datasource_cache();
@@ -414,10 +462,12 @@ BOOST_PYTHON_MODULE(_mapnik2)
     export_line_symbolizer();
     export_line_pattern_symbolizer();
     export_polygon_symbolizer();
+    export_building_symbolizer();
     export_polygon_pattern_symbolizer();
     export_raster_symbolizer();
-    export_text_symbolizer();
+    export_text_placement();
     export_shield_symbolizer();
+    export_debug_symbolizer();
     export_font_engine();
     export_projection();
     export_proj_transform();
@@ -425,18 +475,27 @@ BOOST_PYTHON_MODULE(_mapnik2)
     export_coord();
     export_map();
     export_raster_colorizer();
-    export_glyph_symbolizer();
-    export_inmem_metawriter();
+    export_label_collision_detector();
+    export_logger();
+
+    def("clear_cache", &clear_cache,
+        "\n"
+        "Clear all global caches of markers and mapped memory regions.\n"
+        "\n"
+        "Usage:\n"
+        ">>> from mapnik import clear_cache\n"
+        ">>> clear_cache()\n"
+        );
 
     def("render_grid",&render_grid,
-      ( arg("map"),
-        arg("layer"),
-        args("key")="__id__",
-        arg("resolution")=4,
-        arg("fields")=boost::python::list()
-      )
-    );
-    
+        ( arg("map"),
+          arg("layer"),
+          args("key")="__id__",
+          arg("resolution")=4,
+          arg("fields")=boost::python::list()
+            )
+        );
+
     def("render_to_file",&render_to_file1,
         "\n"
         "Render Map to file using explicit image type.\n"
@@ -478,16 +537,16 @@ BOOST_PYTHON_MODULE(_mapnik2)
         ">>> render_to_file(m,'image.jpeg',scale_factor)\n"
         "\n"
         );
-        
+
     def("render_tile_to_file",&render_tile_to_file,
         "\n"
         "TODO\n"
         "\n"
-        ); 
+        );
+
 
-    
     def("render", &render, render_overloads(
-            "\n" 
+            "\n"
             "Render Map to an AGG image_32 using offsets\n"
             "\n"
             "Usage:\n"
@@ -501,16 +560,29 @@ BOOST_PYTHON_MODULE(_mapnik2)
             ">>> render(m,im,scale_factor)\n"
             ">>> render(m,im,scale_factor,offset[0],offset[1])\n"
             "\n"
-            )); 
+            ));
+
+    def("render_with_detector", &render_with_detector, render_with_detector_overloads(
+            "\n"
+            "Render Map to an AGG image_32 using a pre-constructed detector.\n"
+            "\n"
+            "Usage:\n"
+            ">>> from mapnik import Map, Image, LabelCollisionDetector, render_with_detector, load_map\n"
+            ">>> m = Map(256,256)\n"
+            ">>> load_map(m,'mapfile.xml')\n"
+            ">>> im = Image(m.width,m.height)\n"
+            ">>> detector = LabelCollisionDetector(m)\n"
+            ">>> render_with_detector(m, im, detector)\n"
+            ));
 
     def("render_layer", &render_layer2,
-      (arg("map"),arg("image"),args("layer"))
-    ); 
+        (arg("map"),arg("image"),args("layer"))
+        );
 
     def("render_layer", &mapnik::render_layer_for_grid,
-      (arg("map"),arg("grid"),args("layer"),arg("fields")=boost::python::list())
-    ); 
-    
+        (arg("map"),arg("grid"),args("layer"),arg("fields")=boost::python::list())
+        );
+
 #if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
     def("render",&render3,
         "\n"
@@ -549,7 +621,7 @@ BOOST_PYTHON_MODULE(_mapnik2)
         ">>> from cairo import SVGSurface, Context\n"
         ">>> surface = SVGSurface('image.svg', m.width, m.height)\n"
         ">>> ctx = Context(surface)\n"
-        ">>> load_map(m,'mapfile.xml')\n"  
+        ">>> load_map(m,'mapfile.xml')\n"
         ">>> render(m,context,1,1)\n"
         "\n"
         );
@@ -567,9 +639,69 @@ BOOST_PYTHON_MODULE(_mapnik2)
         ">>> render(m,context)\n"
         "\n"
         );
+
+    def("render_with_detector", &render_with_detector2,
+        "\n"
+        "Render Map to Cairo Context using a pre-constructed detector.\n"
+        "\n"
+        "Usage:\n"
+        ">>> from mapnik import Map, LabelCollisionDetector, render_with_detector, load_map\n"
+        ">>> from cairo import SVGSurface, Context\n"
+        ">>> surface = SVGSurface('image.svg', m.width, m.height)\n"
+        ">>> ctx = Context(surface)\n"
+        ">>> m = Map(256,256)\n"
+        ">>> load_map(m,'mapfile.xml')\n"
+        ">>> detector = LabelCollisionDetector(m)\n"
+        ">>> render_with_detector(m, ctx, detector)\n"
+        );
+
+    def("render_with_detector", &render_with_detector3,
+        "\n"
+        "Render Map to Cairo Context using a pre-constructed detector, scale and offsets.\n"
+        "\n"
+        "Usage:\n"
+        ">>> from mapnik import Map, LabelCollisionDetector, render_with_detector, load_map\n"
+        ">>> from cairo import SVGSurface, Context\n"
+        ">>> surface = SVGSurface('image.svg', m.width, m.height)\n"
+        ">>> ctx = Context(surface)\n"
+        ">>> m = Map(256,256)\n"
+        ">>> load_map(m,'mapfile.xml')\n"
+        ">>> detector = LabelCollisionDetector(m)\n"
+        ">>> render_with_detector(m, ctx, detector, 1, 1, 1)\n"
+        );
+
+    def("render_with_detector", &render_with_detector4,
+        "\n"
+        "Render Map to Cairo Surface using a pre-constructed detector.\n"
+        "\n"
+        "Usage:\n"
+        ">>> from mapnik import Map, LabelCollisionDetector, render_with_detector, load_map\n"
+        ">>> from cairo import SVGSurface, Context\n"
+        ">>> surface = SVGSurface('image.svg', m.width, m.height)\n"
+        ">>> m = Map(256,256)\n"
+        ">>> load_map(m,'mapfile.xml')\n"
+        ">>> detector = LabelCollisionDetector(m)\n"
+        ">>> render_with_detector(m, surface, detector)\n"
+        );
+
+    def("render_with_detector", &render_with_detector5,
+        "\n"
+        "Render Map to Cairo Surface using a pre-constructed detector, scale and offsets.\n"
+        "\n"
+        "Usage:\n"
+        ">>> from mapnik import Map, LabelCollisionDetector, render_with_detector, load_map\n"
+        ">>> from cairo import SVGSurface, Context\n"
+        ">>> surface = SVGSurface('image.svg', m.width, m.height)\n"
+        ">>> m = Map(256,256)\n"
+        ">>> load_map(m,'mapfile.xml')\n"
+        ">>> detector = LabelCollisionDetector(m)\n"
+        ">>> render_with_detector(m, surface, detector, 1, 1, 1)\n"
+        );
+
 #endif
 
     def("scale_denominator", &scale_denominator,
+        (arg("map"),arg("is_geographic")),
         "\n"
         "Return the Map Scale Denominator.\n"
         "Also available as Map.scale_denominator()\n"
@@ -582,7 +714,7 @@ BOOST_PYTHON_MODULE(_mapnik2)
         ">>> scale_denominator(m,Projection(m.srs).geographic)\n"
         "\n"
         );
-   
+
     def("load_map", &load_map, load_map_overloads());
 
     def("load_map_from_string", &load_map_string, load_map_string_overloads());
@@ -603,15 +735,29 @@ BOOST_PYTHON_MODULE(_mapnik2)
   "\n"
   );
 */
-    
+
     def("save_map_to_string", &save_map_to_string, save_map_to_string_overloads());
     def("mapnik_version", &mapnik_version,"Get the Mapnik version number");
-    def("mapnik_svn_revision", &mapnik_svn_revision,"Get the Mapnik svn revision");
+    def("mapnik_version_string", &mapnik_version_string,"Get the Mapnik version string");
     def("has_jpeg", &has_jpeg, "Get jpeg read/write support status");
     def("has_cairo", &has_cairo, "Get cairo library status");
     def("has_pycairo", &has_pycairo, "Get pycairo module status");
 
+    python_optional<mapnik::stroke>();
+    python_optional<mapnik::font_set>();
+    python_optional<mapnik::color>();
+    python_optional<mapnik::box2d<double> >();
+    python_optional<mapnik::composite_mode_e>();
+    python_optional<mapnik::datasource::geometry_t>();
+    python_optional<std::string>();
+    python_optional<unsigned>();
+    python_optional<double>();
+    python_optional<float>();
+    python_optional<bool>();
+    python_optional<int>();
+    python_optional<mapnik::text_transform_e>();
     register_ptr_to_python<mapnik::expression_ptr>();
     register_ptr_to_python<mapnik::path_expression_ptr>();
+    to_python_converter<mapnik::value_holder,mapnik_param_to_python>();
     to_python_converter<mapnik::value,mapnik_value_to_python>();
 }
