@@ -17,7 +17,7 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
-# $Id$
+# 
 
 import os
 from copy import copy
@@ -36,18 +36,16 @@ demo_env = env.Clone()
 demo_env['CXXFLAGS'] = copy(env['LIBMAPNIK_CXXFLAGS'])
 
 if env['HAS_CAIRO']:
-    demo_env.PrependUnique(CPPPATH=env['CAIROMM_CPPPATHS'])
-    demo_env.Append(CXXFLAGS = '-DHAVE_CAIRO')
+    demo_env.PrependUnique(CPPPATH=env['CAIRO_CPPPATHS'])
+    demo_env.Append(CPPDEFINES = '-DHAVE_CAIRO')
 
 libraries =  copy(env['LIBMAPNIK_LIBS'])
-boost_program_options = 'boost_program_options%s' % env['BOOST_APPEND']
-libraries.extend([boost_program_options,'mapnik2'])
+libraries.append('mapnik')
 
 rundemo = demo_env.Program('rundemo', source, LIBS=libraries, LINKFLAGS=env["CUSTOM_LDFLAGS"])
 
 Depends(rundemo, env.subst('../../src/%s' % env['MAPNIK_LIB_NAME']))
 
-# we don't install this app because the datasource paths are relative
-# and we're not going to install the sample data.
-#env.Install(install_prefix + '/bin', rundemo)
-#env.Alias('install', install_prefix + '/bin')
\ No newline at end of file
+# build locally if installing
+if 'install' in COMMAND_LINE_TARGETS:
+    env.Alias('install',rundemo)
