@@ -15,18 +15,19 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id$
+# 
 
 import os
 import glob
 
 Import('env')
 
-# grab all the deja vu fonts
-includes = glob.glob('*/*/*.ttf')
-
-# grab single unifont ttf (available at http://unifoundry.com/unifont.html)
-includes.extend(glob.glob('unifont*.ttf'))
-
-if 'uninstall' not in COMMAND_LINE_TARGETS and not env['SYSTEM_FONTS']:
-    env.Alias(target='install', source=env.Install(env['MAPNIK_FONTS_DEST'], includes))
+if not env['SYSTEM_FONTS']:
+    # grab all the deja vu fonts
+    includes = glob.glob('*/*/*.ttf')
+    # grab single unifont ttf (available at http://unifoundry.com/unifont.html)
+    includes.extend(glob.glob('unifont*.ttf'))
+    target_path = env['MAPNIK_FONTS_DEST']
+    if 'uninstall' not in COMMAND_LINE_TARGETS:
+        env.Alias(target='install', source=env.Install(target_path, includes))
+    env['create_uninstall_target'](env, target_path)
\ No newline at end of file
