@@ -1,8 +1,8 @@
 /*****************************************************************************
- * 
+ *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2006 Artem Pavlenko
+ * Copyright (C) 2011 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,23 +19,23 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *****************************************************************************/
-//$Id: polygon_symbolizer.hpp 39 2005-04-10 20:39:53Z pavlenko $
 
-#ifndef LINE_PATTERN_SYMBOLIZER_HPP
-#define LINE_PATTERN_SYMBOLIZER_HPP
+#ifndef MAPNIK_LINE_PATTERN_SYMBOLIZER_HPP
+#define MAPNIK_LINE_PATTERN_SYMBOLIZER_HPP
 
 // mapnik
+#include <mapnik/config.hpp>
 #include <mapnik/symbolizer.hpp>
-//#include <boost/shared_ptr.hpp>
+#include <mapnik/path_expression.hpp>
 
-namespace mapnik 
-{      
+namespace mapnik
+{
 struct MAPNIK_DECL line_pattern_symbolizer :
         public symbolizer_with_image, public symbolizer_base
 {
     line_pattern_symbolizer(path_expression_ptr file);
     line_pattern_symbolizer(line_pattern_symbolizer const& rhs);
-};    
+};
 }
 
-#endif // LINE_PATTERN_SYMBOLIZER_HPP
+#endif // MAPNIK_LINE_PATTERN_SYMBOLIZER_HPP
