@@ -1,8 +1,8 @@
 /*****************************************************************************
- * 
+ *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2006 Artem Pavlenko
+ * Copyright (C) 2011 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,23 +19,28 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *****************************************************************************/
-//$Id: raster.hpp 39 2005-04-10 20:39:53Z pavlenko $
 
-#ifndef RASTER_HPP
-#define RASTER_HPP
+#ifndef MAPNIK_RASTER_HPP
+#define MAPNIK_RASTER_HPP
 
-#include <mapnik/graphics.hpp>
+// mapnik
+#include <mapnik/box2d.hpp>
+#include <mapnik/image_data.hpp>
+#include <mapnik/noncopyable.hpp>
 
 namespace mapnik {
-struct raster
+class raster : private mapnik::noncopyable
 {
+public:
     box2d<double> ext_;
     image_data_32 data_;
-    raster(box2d<double> const& ext,image_data_32 const& data)
+    bool premultiplied_alpha_;
+    raster(box2d<double> const& ext, unsigned width, unsigned height, bool premultiplied_alpha = false)
         : ext_(ext),
-          data_(data) {}
+          data_(width,height),
+          premultiplied_alpha_(premultiplied_alpha)
+    {}
 };
 }
 
-#endif //RASTER_HPP
-
+#endif // MAPNIK_RASTER_HPP
