@@ -1,8 +1,8 @@
 /*****************************************************************************
- * 
+ *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2006 Artem Pavlenko
+ * Copyright (C) 2011 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,16 +20,15 @@
  *
  *****************************************************************************/
 
-//$Id$
 #ifndef MAPNIK_SCALE_DENOMINATOR_HPP
 #define MAPNIK_SCALE_DENOMINATOR_HPP
 
+// mapnik
 #include <mapnik/config.hpp>
 
 namespace mapnik {
- 
-class Map;
-MAPNIK_DECL double scale_denominator(Map const& map, bool geographic);
+
+MAPNIK_DECL double scale_denominator(double map_scale, bool geographic);
 }
 
 #endif // MAPNIK_SCALE_DENOMINATOR_HPP
