@@ -2,7 +2,7 @@
  *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2006 Artem Pavlenko
+ * Copyright (C) 2011 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,15 +19,25 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *****************************************************************************/
-// $Id$
 
 // mapnik
+#include <mapnik/rule.hpp>
+#include <mapnik/layer.hpp>
+#include <mapnik/feature_type_style.hpp>
+#include <mapnik/debug.hpp>
 #include <mapnik/save_map.hpp>
 #include <mapnik/map.hpp>
 #include <mapnik/ptree_helpers.hpp>
 #include <mapnik/expression_string.hpp>
 #include <mapnik/raster_colorizer.hpp>
-#include <mapnik/metawriter_factory.hpp>
+#include <mapnik/text_placements/simple.hpp>
+#include <mapnik/text_placements/list.hpp>
+#include <mapnik/text_placements/dummy.hpp>
+#include <mapnik/image_compositing.hpp>
+#include <mapnik/image_scaling.hpp>
+#include <mapnik/image_filter.hpp>
+#include <mapnik/image_filter_types.hpp>
+#include <mapnik/parse_path.hpp>
 
 // boost
 #include <boost/algorithm/string.hpp>
@@ -51,7 +61,7 @@ public:
         rule_(r),
         explicit_defaults_(explicit_defaults) {}
 
-    void operator () ( const  point_symbolizer & sym )
+    void operator () ( point_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("PointSymbolizer", ptree()))->second;
@@ -71,36 +81,45 @@ public:
         {
             set_attr( sym_node, "placement", sym.get_point_placement() );
         }
-        add_metawriter_attributes(sym_node, sym);
+        if (sym.get_image_transform())
+        {
+            std::string tr_str = sym.get_image_transform_string();
+            set_attr( sym_node, "transform", tr_str );
+        }
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const line_symbolizer & sym )
+    void operator () ( line_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("LineSymbolizer", ptree()))->second;
 
         const stroke & strk =  sym.get_stroke();
         add_stroke_attributes(sym_node, strk);
-        add_metawriter_attributes(sym_node, sym);
 
         line_symbolizer dfl;
         if ( sym.get_rasterizer() != dfl.get_rasterizer() || explicit_defaults_ )
         {
             set_attr( sym_node, "rasterizer", sym.get_rasterizer() );
         }
+        if ( sym.offset() != dfl.offset() || explicit_defaults_ )
+        {
+            set_attr( sym_node, "offset", sym.offset() );
+        }
+        serialize_symbolizer_base(sym_node, sym);
     }
-        
-    void operator () ( const line_pattern_symbolizer & sym )
+
+    void operator () ( line_pattern_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("LinePatternSymbolizer",
                               ptree()))->second;
 
         add_image_attributes( sym_node, sym );
-        add_metawriter_attributes(sym_node, sym);
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const polygon_symbolizer & sym )
+    void operator () ( polygon_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("PolygonSymbolizer", ptree()))->second;
@@ -118,10 +137,14 @@ public:
         {
             set_attr( sym_node, "gamma", sym.get_gamma() );
         }
-        add_metawriter_attributes(sym_node, sym);
+        if ( sym.get_gamma_method() != dfl.get_gamma_method() || explicit_defaults_ )
+        {
+            set_attr( sym_node, "gamma-method", sym.get_gamma_method() );
+        }
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const polygon_pattern_symbolizer & sym )
+    void operator () ( polygon_pattern_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("PolygonPatternSymbolizer",
@@ -132,32 +155,29 @@ public:
         {
             set_attr( sym_node, "alignment", sym.get_alignment() );
         }
-
         if ( sym.get_gamma() != dfl.get_gamma() || explicit_defaults_ )
         {
             set_attr( sym_node, "gamma", sym.get_gamma() );
         }
-
+        if ( sym.get_gamma_method() != dfl.get_gamma_method() || explicit_defaults_ )
+        {
+            set_attr( sym_node, "gamma-method", sym.get_gamma_method() );
+        }
         add_image_attributes( sym_node, sym );
-        add_metawriter_attributes(sym_node, sym);
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const raster_symbolizer & sym )
+    void operator () ( raster_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("RasterSymbolizer", ptree()))->second;
         raster_symbolizer dfl;
 
-        if ( sym.get_mode() != dfl.get_mode() || explicit_defaults_ )
+        if ( sym.get_scaling_method() != dfl.get_scaling_method() || explicit_defaults_ )
         {
-            set_attr( sym_node, "mode", sym.get_mode() );
+            set_attr( sym_node, "scaling", *scaling_method_to_string(sym.get_scaling_method()) );
         }
 
-        if ( sym.get_scaling() != dfl.get_scaling() || explicit_defaults_ )
-        {
-            set_attr( sym_node, "scaling", sym.get_scaling() );
-        }
-        
         if ( sym.get_opacity() != dfl.get_opacity() || explicit_defaults_ )
         {
             set_attr( sym_node, "opacity", sym.get_opacity() );
@@ -168,66 +188,80 @@ public:
             set_attr( sym_node, "mesh-size", sym.get_mesh_size() );
         }
 
-        if (sym.get_colorizer()) {
+        if (sym.get_colorizer())
+        {
             serialize_raster_colorizer(sym_node, sym.get_colorizer(),
                                        explicit_defaults_);
         }
-        //Note: raster_symbolizer doesn't support metawriters
+
+        boost::optional<bool> premultiplied = sym.premultiplied();
+        if (premultiplied)
+        {
+            set_attr( sym_node, "premultiplied", *sym.premultiplied());
+        }
+
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const shield_symbolizer & sym )
+    void operator () ( shield_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("ShieldSymbolizer",
                               ptree()))->second;
 
-        add_font_attributes( sym_node, sym);
-        add_image_attributes( sym_node, sym);
-        add_metawriter_attributes(sym_node, sym);
+        add_font_attributes(sym_node, sym);
+        add_image_attributes(sym_node, sym);
 
         // pseudo-default-construct a shield_symbolizer. It is used
         // to avoid printing of attributes with default values without
         // repeating the default values here.
         // maybe add a real, explicit default-ctor?
 
-        
-        shield_symbolizer dfl(expression_ptr(), "<no default>", 0, color(0,0,0), path_expression_ptr());
-        
-        if (sym.get_unlock_image() != dfl.get_unlock_image() || explicit_defaults_ )
+        shield_symbolizer dfl;
+
+        if (sym.get_unlock_image() != dfl.get_unlock_image() || explicit_defaults_)
         {
-            set_attr( sym_node, "unlock-image", sym.get_unlock_image() );
+            set_attr(sym_node, "unlock-image", sym.get_unlock_image());
         }
-        if (sym.get_no_text() != dfl.get_no_text() || explicit_defaults_ )
+
+        if (sym.get_placement_options()->defaults.format.text_opacity !=
+                dfl.get_placement_options()->defaults.format.text_opacity || explicit_defaults_)
         {
-            set_attr( sym_node, "no-text", sym.get_no_text() );
-        }
-        if (sym.get_text_opacity() != dfl.get_text_opacity() || explicit_defaults_ )
-        {
-            set_attr( sym_node, "text-opacity", sym.get_text_opacity() );
+            set_attr(sym_node, "text-opacity", sym.get_placement_options()->defaults.format.text_opacity);
         }
         position displacement = sym.get_shield_displacement();
-        if ( displacement.get<0>() != dfl.get_shield_displacement().get<0>() || explicit_defaults_ )
+        if (displacement.first != dfl.get_shield_displacement().first || explicit_defaults_)
         {
-            set_attr( sym_node, "shield-dx", displacement.get<0>() );
+            set_attr(sym_node, "shield-dx", displacement.first);
         }
-        if ( displacement.get<1>() != dfl.get_shield_displacement().get<1>() || explicit_defaults_ )
+        if (displacement.second != dfl.get_shield_displacement().second || explicit_defaults_)
         {
-            set_attr( sym_node, "shield-dy", displacement.get<1>() );
+            set_attr(sym_node, "shield-dy", displacement.second);
         }
-
+        if (sym.get_image_transform())
+        {
+            std::string tr_str = sym.get_image_transform_string();
+            set_attr( sym_node, "transform", tr_str );
+        }
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( const text_symbolizer & sym )
+    void operator () ( text_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("TextSymbolizer",
                               ptree()))->second;
 
         add_font_attributes( sym_node, sym);
-        add_metawriter_attributes(sym_node, sym);
+        serialize_symbolizer_base(sym_node, sym);
+        text_symbolizer dfl;
+        if (sym.get_halo_rasterizer() != dfl.get_halo_rasterizer() || explicit_defaults_)
+        {
+            set_attr(sym_node, "halo-rasterizer", sym.get_halo_rasterizer());
+        }
     }
 
-    void operator () ( const building_symbolizer & sym )
+    void operator () ( building_symbolizer const& sym )
     {
         ptree & sym_node = rule_.push_back(
             ptree::value_type("BuildingSymbolizer", ptree()))->second;
@@ -241,11 +275,11 @@ public:
         {
             set_attr( sym_node, "fill-opacity", sym.get_opacity() );
         }
-        if ( sym.height() != dfl.height() || explicit_defaults_ )
+        if (sym.height())
         {
-            set_attr( sym_node, "height", sym.height() );
+            set_attr( sym_node, "height", mapnik::to_expression_string(*sym.height()) );
         }
-        add_metawriter_attributes(sym_node, sym);
+        serialize_symbolizer_base(sym_node, sym);
     }
 
     void operator () ( markers_symbolizer const& sym)
@@ -253,14 +287,19 @@ public:
         ptree & sym_node = rule_.push_back(
             ptree::value_type("MarkersSymbolizer", ptree()))->second;
         markers_symbolizer dfl(parse_path("")); //TODO: Parameter?
-        std::string const& filename = path_processor_type::to_string( *sym.get_filename());
-        if ( ! filename.empty() ) {
+        if (sym.get_filename())
+        {
+            std::string filename = path_processor_type::to_string(*sym.get_filename());
             set_attr( sym_node, "file", filename );
         }
         if (sym.get_allow_overlap() != dfl.get_allow_overlap() || explicit_defaults_)
         {
             set_attr( sym_node, "allow-overlap", sym.get_allow_overlap() );
         }
+        if (sym.get_ignore_placement() != dfl.get_ignore_placement() || explicit_defaults_)
+        {
+            set_attr( sym_node, "ignore-placement", sym.get_ignore_placement() );
+        }
         if (sym.get_spacing() != dfl.get_spacing() || explicit_defaults_)
         {
             set_attr( sym_node, "spacing", sym.get_spacing() );
@@ -273,135 +312,87 @@ public:
         {
             set_attr( sym_node, "fill", sym.get_fill() );
         }
+        if (sym.get_fill_opacity() != dfl.get_fill_opacity() || explicit_defaults_)
+        {
+            set_attr( sym_node, "fill-opacity", sym.get_fill_opacity() );
+        }
         if (sym.get_opacity() != dfl.get_opacity() || explicit_defaults_)
         {
             set_attr( sym_node, "opacity", sym.get_opacity() );
         }
         if (sym.get_width() != dfl.get_width() || explicit_defaults_)
         {
-            set_attr( sym_node, "width", sym.get_width() );
+            set_attr( sym_node, "width", to_expression_string(*sym.get_width()) );
         }
         if (sym.get_height() != dfl.get_height() || explicit_defaults_)
         {
-            set_attr( sym_node, "height", sym.get_height() );
-        }
-        if (sym.get_marker_type() != dfl.get_marker_type() || explicit_defaults_)
-        {
-            set_attr( sym_node, "marker-type", sym.get_marker_type() );
+            set_attr( sym_node, "height", to_expression_string(*sym.get_height()) );
         }
         if (sym.get_marker_placement() != dfl.get_marker_placement() || explicit_defaults_)
         {
             set_attr( sym_node, "placement", sym.get_marker_placement() );
         }
-        std::string tr_str = sym.get_transform_string();
-        if (tr_str != "matrix(1, 0, 0, 1, 0, 0)" || explicit_defaults_ )
+        if ( sym.get_marker_multi_policy() != dfl.get_marker_multi_policy() || explicit_defaults_ )
         {
+            set_attr( sym_node, "multi-policy", sym.get_marker_multi_policy() );
+        }
+        if (sym.get_image_transform())
+        {
+            std::string tr_str = sym.get_image_transform_string();
             set_attr( sym_node, "transform", tr_str );
         }
 
-        const stroke & strk =  sym.get_stroke();
-        add_stroke_attributes(sym_node, strk);
+        boost::optional<stroke> const& strk = sym.get_stroke();
+        if (strk)
+        {
+            add_stroke_attributes(sym_node, *strk);
+        }
 
-        add_metawriter_attributes(sym_node, sym);
+        serialize_symbolizer_base(sym_node, sym);
     }
 
-    void operator () ( glyph_symbolizer const& sym)
+    template <typename Symbolizer>
+    void operator () ( Symbolizer const& sym)
     {
-        ptree &node = rule_.push_back(
-            ptree::value_type("GlyphSymbolizer", ptree())
-            )->second;
-                
-        glyph_symbolizer dfl("<no default>", expression_ptr());
-
-        // face_name
-        set_attr( node, "face-name", sym.get_face_name() );    
-
-        // char
-        if (sym.get_char()) {
-            const std::string &str =
-                to_expression_string(*sym.get_char());
-            set_attr( node, "char", str );
-        }
-
-        // angle
-        if (sym.get_angle()) {
-            const std::string &str =
-                to_expression_string(*sym.get_angle());
-            set_attr( node, "angle", str );
-        }
-
-        // value
-        if (sym.get_value()) {
-            const std::string &str =
-                to_expression_string(*sym.get_value());
-            set_attr( node, "value", str );
-        }
-
-        // size
-        if (sym.get_size()) {
-            const std::string &str =
-                to_expression_string(*sym.get_size());
-            set_attr( node, "size", str );
-        }
-
-        // color
-        if (sym.get_color()) {
-            const std::string &str =
-                to_expression_string(*sym.get_color());
-            set_attr( node, "color", str );
-        }
-
-        // colorizer
-        if (sym.get_colorizer()) {
-            serialize_raster_colorizer(node, sym.get_colorizer(),
-                                       explicit_defaults_);
-        }
-
-        // allow_overlap
-        if (sym.get_allow_overlap() != dfl.get_allow_overlap() || explicit_defaults_ )
-        {
-            set_attr( node, "allow-overlap", sym.get_allow_overlap() );
-        }
-        // avoid_edges
-        if (sym.get_avoid_edges() != dfl.get_avoid_edges() || explicit_defaults_ )
-        {
-            set_attr( node, "avoid-edges", sym.get_avoid_edges() );
-        }
-
-        // displacement
-        position displacement = sym.get_displacement();
-        if ( displacement.get<0>() != dfl.get_displacement().get<0>() || explicit_defaults_ )
-        {
-            set_attr( node, "dx", displacement.get<0>() );
-        }
-        if ( displacement.get<1>() != dfl.get_displacement().get<1>() || explicit_defaults_ )
-        {
-            set_attr( node, "dy", displacement.get<1>() );
-        }
-
-        // halo fill & radius
-        const color & c = sym.get_halo_fill();
-        if ( c != dfl.get_halo_fill() || explicit_defaults_ )
-        {
-            set_attr( node, "halo-fill", c );
-        }
-        
-        if (sym.get_halo_radius() != dfl.get_halo_radius() || explicit_defaults_ )
-        {
-            set_attr( node, "halo-radius", sym.get_halo_radius() );
-        }
-
-        // angle_mode
-        if (sym.get_angle_mode() != dfl.get_angle_mode() || explicit_defaults_ )
-        {
-            set_attr( node, "angle-mode", sym.get_angle_mode() );
-        }
-        add_metawriter_attributes(node, sym);
+        // not-supported
+#ifdef MAPNIK_DEBUG
+        MAPNIK_LOG_WARN(save_map) << typeid(sym).name() << " is not supported";
+#endif
     }
 
 private:
     serialize_symbolizer();
-    
+
+    void serialize_symbolizer_base(ptree & node, symbolizer_base const& sym)
+    {
+        symbolizer_base dfl = symbolizer_base();
+        if (sym.get_transform())
+        {
+            std::string tr_str = sym.get_transform_string();
+            set_attr( node, "geometry-transform", tr_str );
+        }
+        if (sym.clip() != dfl.clip() || explicit_defaults_)
+        {
+            set_attr( node, "clip", sym.clip() );
+        }
+        if (sym.simplify_algorithm() != dfl.simplify_algorithm() || explicit_defaults_)
+        {
+            set_attr( node, "simplify-algorithm", *simplify_algorithm_to_string(sym.simplify_algorithm()) );
+        }
+        if (sym.simplify_tolerance() != dfl.simplify_tolerance() || explicit_defaults_)
+        {
+            set_attr( node, "simplify", sym.simplify_tolerance() );
+        }
+        if (sym.smooth() != dfl.smooth() || explicit_defaults_)
+        {
+            set_attr( node, "smooth", sym.smooth() );
+        }
+        if (sym.comp_op() != dfl.comp_op() || explicit_defaults_)
+        {
+            set_attr( node, "comp-op", *comp_op_to_string(sym.comp_op()) );
+        }
+    }
+
     void serialize_raster_colorizer(ptree & sym_node,
                                     raster_colorizer_ptr const& colorizer,
                                     bool explicit_defaults)
@@ -425,164 +416,53 @@ private:
         }
 
     }
-    
-    void add_image_attributes(ptree & node, const symbolizer_with_image & sym)
+
+    void add_image_attributes(ptree & node, symbolizer_with_image const& sym)
     {
-        std::string const& filename = path_processor_type::to_string( *sym.get_filename());
-        if ( ! filename.empty() ) {
+        if (sym.get_filename())
+        {
+            std::string filename = path_processor_type::to_string( *sym.get_filename());
             set_attr( node, "file", filename );
         }
         if (sym.get_opacity() != 1.0 || explicit_defaults_ )
         {
             set_attr( node, "opacity", sym.get_opacity() );
         }
-        
-        std::string tr_str = sym.get_transform_string();
-        if (tr_str != "matrix(1, 0, 0, 1, 0, 0)" || explicit_defaults_ )
-        {
-            set_attr( node, "transform", tr_str );
-        }
-
     }
+
     void add_font_attributes(ptree & node, const text_symbolizer & sym)
     {
-        expression_ptr const& expr = sym.get_name();
-        const std::string & name = to_expression_string(*expr);
-                
-        if (!name.empty()) {
-            ptree& text_node = node.push_back(ptree::value_type("<xmltext>", ptree()))->second;
-            text_node.put_value(name);
+        text_placements_ptr p = sym.get_placement_options();
+        p->defaults.to_xml(node, explicit_defaults_);
+        /* Known types:
+           - text_placements_dummy: no handling required
+           - text_placements_simple: positions string
+           - text_placements_list: list string
+        */
+        text_placements_simple *simple = dynamic_cast<text_placements_simple *>(p.get());
+        text_placements_list *list = dynamic_cast<text_placements_list *>(p.get());
+        if (simple) {
+            set_attr(node, "placement-type", "simple");
+            set_attr(node, "placements", simple->get_positions());
         }
-        const std::string & face_name = sym.get_face_name();
-        if ( ! face_name.empty() ) {
-            set_attr( node, "face-name", face_name );
-        }
-        const std::string & fontset_name = sym.get_fontset().get_name();
-        if ( ! fontset_name.empty() ) {
-            set_attr( node, "fontset-name", fontset_name );
-        }
-
-        set_attr( node, "size", sym.get_text_size() );
-        set_attr( node, "fill", sym.get_fill() );
-
-        // pseudo-default-construct a text_symbolizer. It is used
-        // to avoid printing ofattributes with default values without
-        // repeating the default values here.
-        // maybe add a real, explicit default-ctor?
-        // FIXME
-        text_symbolizer dfl(expression_ptr(), "<no default>",
-                            0, color(0,0,0) );
-
-        position displacement = sym.get_displacement();
-        if ( displacement.get<0>() != dfl.get_displacement().get<0>() || explicit_defaults_ )
-        {
-            set_attr( node, "dx", displacement.get<0>() );
-        }
-        if ( displacement.get<1>() != dfl.get_displacement().get<1>() || explicit_defaults_ )
-        {
-            set_attr( node, "dy", displacement.get<1>() );
-        }
-
-        if (sym.get_label_placement() != dfl.get_label_placement() || explicit_defaults_ )
-        {
-            set_attr( node, "placement", sym.get_label_placement() );
-        }
-
-        if (sym.get_vertical_alignment() != dfl.get_vertical_alignment() || explicit_defaults_ )
-        {
-            set_attr( node, "vertical-alignment", sym.get_vertical_alignment() );
-        }
-
-        if (sym.get_halo_radius() != dfl.get_halo_radius() || explicit_defaults_ )
-        {
-            set_attr( node, "halo-radius", sym.get_halo_radius() );
-        }
-        const color & c = sym.get_halo_fill();
-        if ( c != dfl.get_halo_fill() || explicit_defaults_ )
-        {
-            set_attr( node, "halo-fill", c );
-        }
-        if (sym.get_text_ratio() != dfl.get_text_ratio() || explicit_defaults_ )
-        {
-            set_attr( node, "text-ratio", sym.get_text_ratio() );
-        }
-        if (sym.get_wrap_width() != dfl.get_wrap_width() || explicit_defaults_ )
-        {
-            set_attr( node, "wrap-width", sym.get_wrap_width() );
-        }
-        if (sym.get_wrap_before() != dfl.get_wrap_before() || explicit_defaults_ )
-        {
-            set_attr( node, "wrap-before", sym.get_wrap_before() );
-        }
-        if (sym.get_wrap_char() != dfl.get_wrap_char() || explicit_defaults_ )
-        {
-            set_attr( node, "wrap-character", std::string(1, sym.get_wrap_char()) );
-        }
-        if (sym.get_text_transform() != dfl.get_text_transform() || explicit_defaults_ )
-        {
-            set_attr( node, "text-transform", sym.get_text_transform() );
-        }
-        if (sym.get_line_spacing() != dfl.get_line_spacing() || explicit_defaults_ )
-        {
-            set_attr( node, "line-spacing", sym.get_line_spacing() );
-        }
-        if (sym.get_character_spacing() != dfl.get_character_spacing() || explicit_defaults_ )
-        {
-            set_attr( node, "character-spacing", sym.get_character_spacing() );
-        }
-        if (sym.get_label_position_tolerance() != dfl.get_label_position_tolerance() || explicit_defaults_ )
-        {
-            set_attr( node, "label-position-tolerance", sym.get_label_position_tolerance() );
-        }
-        if (sym.get_label_spacing() != dfl.get_label_spacing() || explicit_defaults_ )
-        {
-            set_attr( node, "spacing", sym.get_label_spacing() );
-        }
-        if (sym.get_minimum_distance() != dfl.get_minimum_distance() || explicit_defaults_ )
-        {
-            set_attr( node, "minimum-distance", sym.get_minimum_distance() );
-        }
-        if (sym.get_minimum_padding() != dfl.get_minimum_padding() || explicit_defaults_ )
-        {
-            set_attr( node, "minimum-padding", sym.get_minimum_padding() );
-        }
-        if (sym.get_minimum_path_length() != dfl.get_minimum_path_length() || explicit_defaults_ )
-        {
-            set_attr( node, "minimum-path-length", sym.get_minimum_path_length() );
-        }
-        if (sym.get_allow_overlap() != dfl.get_allow_overlap() || explicit_defaults_ )
-        {
-            set_attr( node, "allow-overlap", sym.get_allow_overlap() );
-        }
-        if (sym.get_avoid_edges() != dfl.get_avoid_edges() || explicit_defaults_ )
-        {
-            set_attr( node, "avoid-edges", sym.get_avoid_edges() );
-        }
-        // for shield_symbolizer this is later overridden
-        if (sym.get_text_opacity() != dfl.get_text_opacity() || explicit_defaults_ )
-        {
-            set_attr( node, "opacity", sym.get_text_opacity() );
-        }
-        if (sym.get_max_char_angle_delta() != dfl.get_max_char_angle_delta() || explicit_defaults_ )
-        {
-            set_attr( node, "max-char-angle-delta", sym.get_max_char_angle_delta() );
-        }
-        if (sym.get_horizontal_alignment() != dfl.get_horizontal_alignment() || explicit_defaults_ )
-        {
-            set_attr( node, "horizontal-alignment", sym.get_horizontal_alignment() );
-        }
-        if (sym.get_justify_alignment() != dfl.get_justify_alignment() || explicit_defaults_ )
-        {
-            set_attr( node, "justify-alignment", sym.get_justify_alignment() );
+        if (list) {
+            set_attr(node, "placement-type", "list");
+            unsigned i;
+            //dfl = last properties passed as default so only attributes that change are actually written
+            text_symbolizer_properties *dfl = &(list->defaults);
+            for (i=0; i < list->size(); i++) {
+                ptree &placement_node = node.push_back(ptree::value_type("Placement", ptree()))->second;
+                list->get(i).to_xml(placement_node, explicit_defaults_, *dfl);
+                dfl = &(list->get(i));
+            }
         }
     }
 
-
     void add_stroke_attributes(ptree & node, const stroke & strk)
     {
 
         stroke dfl = stroke();
-        
+
         if ( strk.get_color() != dfl.get_color() || explicit_defaults_ )
         {
             set_attr( node, "stroke", strk.get_color() );
@@ -607,9 +487,13 @@ private:
         {
             set_attr( node, "stroke-gamma", strk.get_gamma());
         }
+        if ( strk.get_gamma_method() != dfl.get_gamma_method() || explicit_defaults_ )
+        {
+            set_attr( node, "stroke-gamma-method", strk.get_gamma_method() );
+        }
         if ( strk.dash_offset() != dfl.dash_offset() || explicit_defaults_ )
         {
-            set_attr( node, "stroke-dash-offset", strk.dash_offset());
+            set_attr( node, "stroke-dashoffset", strk.dash_offset());
         }
         if ( ! strk.get_dash_array().empty() )
         {
@@ -621,16 +505,6 @@ private:
             }
             set_attr( node, "stroke-dasharray", os.str() );
         }
-                
-    }
-    void add_metawriter_attributes(ptree &node, symbolizer_base const& sym)
-    {
-        if (!sym.get_metawriter_name().empty() || explicit_defaults_) {
-            set_attr(node, "meta-writer", sym.get_metawriter_name());
-        }
-        if (!sym.get_metawriter_properties_overrides().empty() || explicit_defaults_) {
-            set_attr(node, "meta-output", sym.get_metawriter_properties_overrides().to_string());
-        }
     }
 
     ptree & rule_;
@@ -647,10 +521,6 @@ void serialize_rule( ptree & style_node, const rule & r, bool explicit_defaults)
     {
         set_attr(rule_node, "name", r.get_name());
     }
-    if ( r.get_title() != dfl.get_title() )
-    {
-        set_attr(rule_node, "title", r.get_title());
-    }
 
     if ( r.has_else_filter() )
     {
@@ -668,7 +538,7 @@ void serialize_rule( ptree & style_node, const rule & r, bool explicit_defaults)
         expression_ptr const& expr = r.get_filter();
         std::string filter = mapnik::to_expression_string(*expr);
         std::string default_filter = mapnik::to_expression_string(*dfl.get_filter());
-            
+
         if ( filter != default_filter)
         {
             rule_node.push_back( ptree::value_type(
@@ -698,21 +568,57 @@ void serialize_rule( ptree & style_node, const rule & r, bool explicit_defaults)
 
 void serialize_style( ptree & map_node, Map::const_style_iterator style_it, bool explicit_defaults )
 {
-    const feature_type_style & style = style_it->second;
-    const std::string & name = style_it->first;
-    filter_mode_e filter_mode = style.get_filter_mode();
+    feature_type_style const& style = style_it->second;
+    std::string const& name = style_it->first;
 
     ptree & style_node = map_node.push_back(
         ptree::value_type("Style", ptree()))->second;
 
     set_attr(style_node, "name", name);
-    
+
     feature_type_style dfl;
+    filter_mode_e filter_mode = style.get_filter_mode();
     if (filter_mode != dfl.get_filter_mode() || explicit_defaults)
     {
         set_attr(style_node, "filter-mode", filter_mode);
     }
 
+    double opacity = style.get_opacity();
+    if (opacity != dfl.get_opacity() || explicit_defaults)
+    {
+        set_attr(style_node, "opacity", opacity);
+    }
+
+    boost::optional<composite_mode_e> comp_op = style.comp_op();
+    if (comp_op)
+    {
+        set_attr(style_node, "comp-op", *comp_op_to_string(*comp_op));
+    }
+    else if (explicit_defaults)
+    {
+        set_attr(style_node, "comp-op", "src-over");
+    }
+
+    if (style.image_filters().size() > 0)
+    {
+        std::string filters_str;
+        std::back_insert_iterator<std::string> sink(filters_str);
+        if (generate_image_filters(sink, style.image_filters()))
+        {
+            set_attr(style_node, "image-filters", filters_str);
+        }
+    }
+
+    if (style.direct_image_filters().size() > 0)
+    {
+        std::string filters_str;
+        std::back_insert_iterator<std::string> sink(filters_str);
+        if (generate_image_filters(sink, style.direct_image_filters()))
+        {
+            set_attr(style_node, "direct-image-filters", filters_str);
+        }
+    }
+
     rules::const_iterator it = style.get_rules().begin();
     rules::const_iterator end = style.get_rules().end();
     for (; it != end; ++it)
@@ -724,8 +630,8 @@ void serialize_style( ptree & map_node, Map::const_style_iterator style_it, bool
 
 void serialize_fontset( ptree & map_node, Map::const_fontset_iterator fontset_it )
 {
-    const font_set & fontset = fontset_it->second;
-    const std::string & name = fontset_it->first;
+    font_set const& fontset = fontset_it->second;
+    std::string const& name = fontset_it->first;
 
     ptree & fontset_node = map_node.push_back(
         ptree::value_type("FontSet", ptree()))->second;
@@ -761,60 +667,122 @@ void serialize_datasource( ptree & layer_node, datasource_ptr datasource)
     }
 }
 
+class serialize_type : public boost::static_visitor<>
+{
+public:
+    serialize_type( boost::property_tree::ptree & node):
+        node_(node) {}
+
+    void operator () ( mapnik::value_integer val ) const
+    {
+        node_.put("<xmlattr>.type", "int" );
+    }
+
+    void operator () ( mapnik::value_double val ) const
+    {
+        node_.put("<xmlattr>.type", "float" );
+    }
+
+    void operator () ( std::string const& val ) const
+    {
+        node_.put("<xmlattr>.type", "string" );
+    }
+
+    void operator () ( mapnik::value_null val ) const
+    {
+        node_.put("<xmlattr>.type", "string" );
+    }
+
+private:
+    boost::property_tree::ptree & node_;
+};
+
+void serialize_parameters( ptree & map_node, mapnik::parameters const& params)
+{
+    if (params.size()) {
+        ptree & params_node = map_node.push_back(
+            ptree::value_type("Parameters", ptree()))->second;
+
+        parameters::const_iterator it = params.begin();
+        parameters::const_iterator end = params.end();
+        for (; it != end; ++it)
+        {
+            boost::property_tree::ptree & param_node = params_node.push_back(
+                boost::property_tree::ptree::value_type("Parameter",
+                                                        boost::property_tree::ptree()))->second;
+            param_node.put("<xmlattr>.name", it->first );
+            param_node.put_value( it->second );
+            boost::apply_visitor(serialize_type(param_node),it->second);
+        }
+    }
+}
+
 void serialize_layer( ptree & map_node, const layer & layer, bool explicit_defaults )
 {
     ptree & layer_node = map_node.push_back(
         ptree::value_type("Layer", ptree()))->second;
+
     if ( layer.name() != "" )
     {
         set_attr( layer_node, "name", layer.name() );
     }
 
-    if ( layer.abstract() != "" )
-    {
-        set_attr( layer_node, "abstract", layer.abstract() );
-    }
-
-    if ( layer.title() != "" )
-    {
-        set_attr( layer_node, "title", layer.title() );
-    }
-
     if ( layer.srs() != "" )
     {
         set_attr( layer_node, "srs", layer.srs() );
     }
 
-    if ( !layer.isActive() || explicit_defaults )
+    if ( !layer.active() || explicit_defaults )
     {
-        set_attr/*<bool>*/( layer_node, "status", layer.isActive() );
+        set_attr/*<bool>*/( layer_node, "status", layer.active() );
     }
-        
+
     if ( layer.clear_label_cache() || explicit_defaults )
-    {        
+    {
         set_attr/*<bool>*/( layer_node, "clear-label-cache", layer.clear_label_cache() );
     }
 
-    if ( layer.getMinZoom() )
+    if ( layer.min_zoom() )
     {
-        set_attr( layer_node, "minzoom", layer.getMinZoom() );
+        set_attr( layer_node, "minzoom", layer.min_zoom() );
     }
 
-    if ( layer.getMaxZoom() != std::numeric_limits<double>::max() )
+    if ( layer.max_zoom() != std::numeric_limits<double>::max() )
     {
-        set_attr( layer_node, "maxzoom", layer.getMaxZoom() );
+        set_attr( layer_node, "maxzoom", layer.max_zoom() );
     }
 
-    if ( layer.isQueryable() || explicit_defaults )
+    if ( layer.queryable() || explicit_defaults )
     {
-        set_attr( layer_node, "queryable", layer.isQueryable() );
+        set_attr( layer_node, "queryable", layer.queryable() );
     }
 
     if ( layer.cache_features() || explicit_defaults )
-    {        
+    {
         set_attr/*<bool>*/( layer_node, "cache-features", layer.cache_features() );
     }
 
+    if ( layer.group_by() != "" || explicit_defaults )
+    {
+        set_attr( layer_node, "group-by", layer.group_by() );
+    }
+
+    boost::optional<int> const& buffer_size = layer.buffer_size();
+    if ( buffer_size || explicit_defaults)
+    {
+        set_attr( layer_node, "buffer-size", *buffer_size );
+    }
+
+    optional<box2d<double> > const& maximum_extent = layer.maximum_extent();
+    if ( maximum_extent)
+    {
+        std::ostringstream s;
+        s << std::setprecision(16)
+          << maximum_extent->minx() << "," << maximum_extent->miny() << ","
+          << maximum_extent->maxx() << "," << maximum_extent->maxy();
+        set_attr( layer_node, "maximum-extent", s.str() );
+    }
+
     std::vector<std::string> const& style_names = layer.styles();
     for (unsigned i = 0; i < style_names.size(); ++i)
     {
@@ -831,18 +799,6 @@ void serialize_layer( ptree & map_node, const layer & layer, bool explicit_defau
     }
 }
 
-void serialize_metawriter(ptree & map_node, Map::const_metawriter_iterator metawriter_it, bool explicit_defaults)
-{
-    std::string const& name = metawriter_it->first;
-    metawriter_ptr const& metawriter = metawriter_it->second;
-
-    ptree & metawriter_node = map_node.push_back(
-        ptree::value_type("MetaWriter", ptree()))->second;
-
-    set_attr(metawriter_node, "name", name);
-    metawriter_save(metawriter, metawriter_node, explicit_defaults);
-}
-
 void serialize_map(ptree & pt, Map const & map, bool explicit_defaults)
 {
 
@@ -861,17 +817,17 @@ void serialize_map(ptree & pt, Map const & map, bool explicit_defaults)
     {
         set_attr( map_node, "background-image", *image_filename );
     }
-    
-    unsigned buffer_size = map.buffer_size();
+
+    int buffer_size = map.buffer_size();
     if ( buffer_size || explicit_defaults)
     {
-        set_attr( map_node, "buffer-size", buffer_size ); 
+        set_attr( map_node, "buffer-size", buffer_size );
     }
 
     std::string const& base_path = map.base_path();
     if ( !base_path.empty() || explicit_defaults)
     {
-        set_attr( map_node, "base", base_path ); 
+        set_attr( map_node, "base", base_path );
     }
 
     optional<box2d<double> > const& maximum_extent = map.maximum_extent();
@@ -881,7 +837,7 @@ void serialize_map(ptree & pt, Map const & map, bool explicit_defaults)
         s << std::setprecision(16)
           << maximum_extent->minx() << "," << maximum_extent->miny() << ","
           << maximum_extent->maxx() << "," << maximum_extent->maxy();
-        set_attr( map_node, "maximum-extent", s.str() ); 
+        set_attr( map_node, "maximum-extent", s.str() );
     }
 
     {
@@ -893,13 +849,7 @@ void serialize_map(ptree & pt, Map const & map, bool explicit_defaults)
         }
     }
 
-    parameters extra_attr = map.get_extra_attributes();
-    parameters::const_iterator p_it = extra_attr.begin();
-    parameters::const_iterator p_end = extra_attr.end();
-    for (; p_it != p_end; ++p_it)
-    {
-        set_attr( map_node, p_it->first, p_it->second ); 
-    }
+    serialize_parameters( map_node, map.get_extra_parameters());
 
     Map::const_style_iterator it = map.styles().begin();
     Map::const_style_iterator end = map.styles().end();
@@ -913,12 +863,6 @@ void serialize_map(ptree & pt, Map const & map, bool explicit_defaults)
     {
         serialize_layer( map_node, layers[i], explicit_defaults );
     }
-
-    Map::const_metawriter_iterator m_it = map.begin_metawriters();
-    Map::const_metawriter_iterator m_end = map.end_metawriters();
-    for (; m_it != m_end; ++m_it) {
-        serialize_metawriter(map_node, m_it, explicit_defaults);
-    }
 }
 
 void save_map(Map const & map, std::string const& filename, bool explicit_defaults)
