@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 
 from nose.tools import *
-
-import mapnik2, pickle
+from utilities import execution_path, run_all
+import mapnik
 
 # Tests that exercise fonts. 
 
@@ -10,7 +10,7 @@ import mapnik2, pickle
 # Todo: Add logic to use this TextSymbolizer in a rendering
 #@raises(UserWarning)
 #def test_invalid_font():
-#    ts = mapnik2.TextSymbolizer('Name', 'Invalid Font Name', int(8), mapnik2.Color('black'))
+#    ts = mapnik.TextSymbolizer('Name', 'Invalid Font Name', int(8), mapnik.Color('black'))
 
 if __name__ == "__main__":
-    [eval(run)() for run in dir() if 'test_' in run]
+    run_all(eval(x) for x in dir() if x.startswith("test_"))
